#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;

namespace HeuristicLab.Core.Networks {
  [Item("NetworkItem", "Abstract base class for items of a network.")]
  [StorableClass]
  public abstract class NetworkItem : NamedItem, INetworkItem {
    public static readonly string PathSeparator = " => ";
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Class; }
    }

    private INetworkItem parent;
    public INetworkItem Parent {
      get { return parent; }
      protected set {
        if (value != parent) {
          DeregisterParentEvents();
          parent = value;
          RegisterParentEvents();
          OnPathChanged();
        }
      }
    }
    public virtual IEnumerable<INetworkItem> Children {
      get { return Enumerable.Empty<INetworkItem>(); }
    }
    public string Path {
      get {
        return parent != null ? parent.Path + PathSeparator + Name : Name;
      }
    }

    [StorableConstructor]
    protected NetworkItem(bool deserializing) : base(deserializing) { }
    protected NetworkItem(NetworkItem original, Cloner cloner) : base(original, cloner) { }
    protected NetworkItem() : base("NetworkItem") { }
    protected NetworkItem(string name) : base(name) { }
    protected NetworkItem(string name, string description) : base(name, description) { }

    protected override void OnNameChanging(CancelEventArgs<string> e) {
      base.OnNameChanging(e);
      e.Cancel = e.Cancel || e.Value.Contains(PathSeparator);
    }
    protected override void OnNameChanged() {
      base.OnNameChanged();
      OnPathChanged();
    }

    public event EventHandler PathChanged;
    protected virtual void OnPathChanged() {
      var handler = PathChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    #region Parent Events
    protected virtual void RegisterParentEvents() {
      if (parent != null) {
        parent.PathChanged += Parent_PathChanged;
      }
    }
    protected virtual void DeregisterParentEvents() {
      if (parent != null) {
        parent.PathChanged -= Parent_PathChanged;
      }
    }
    void Parent_PathChanged(object sender, EventArgs e) {
      OnPathChanged();
    }
    #endregion
  }
}
